////////////////////////////////////////////////////////////////////////////////////////
// Voronoi generation and handling class.
// Uses conic intersections between point areas to generate the Voronoi diagram.
// Based on example code by Nate Robins 1997.
// Written by Vladeta Stojanovic.
//
// Version: 270612_01
////////////////////////////////////////////////////////////////////////////////////////

#ifndef VORONOI_H_
#define VORONOI_H_

#include <glut.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctime> 
#include <math.h>
#include <FreeImage.h>

struct HEIGHT_DATA
{
	unsigned char* m_Data;
	unsigned char* targetDataX;
	unsigned char* targetDataY;
	int m_Size_;
};

typedef struct _node 
{
    double x, y;
    unsigned char r, g, b;
    struct _node* next;
} node;

class Voronoi
{
public:

	Voronoi();
	~Voronoi();

	//Data loading member functions
	void LoadBMP(char * file_name, int size);
	bool LoadRAWMap(char * file_name, int _size);
	bool UnloadRAWMap();

	//Voronoi topology generation
	void Init();
	void InitProjection();
	void GenerateAbsTarget(int absValue);
	void GenerateAbs(int absValue, bool outside, int stepSize);
	void GenerateRange(int minValue, int maxValue, int stepSize);
	void DrawPoint(int _size, bool drawPoints);

	void GenerateScreenshot(char * file_name, int width, int height);

	int m_Size;
	int m_TotalPoints;

private:

	node  *mPoints;
	HEIGHT_DATA m_heightData; //map height data
};

extern Voronoi *gVoronoi;

#endif
